@@ -16,8 +16,9 @@
 
 import os
 import urllib
+import time
 
-from trac.config import Option, IntOption
+from trac.config import BoolOption, IntOption, Option
 from trac.core import *
 from trac.db.pool import ConnectionPool
 from trac.util.text import unicode_passwd
@@ -35,18 +36,26 @@ class IDatabaseConnector(Interface):
 
     def get_supported_schemes():
         """Return the connection URL schemes supported by the connector, and
-        their relative priorities as an iterable of `(scheme, priority)` tuples.
+        their relative priorities as an iterable of `(scheme, priority)`
+        tuples.
+        
+        If `priority` is a negative number, this is indicative of an
+        error  condition with the connector. An error message should be 
+        attached to the `error` attribute of the connector.
         """
 
-    def get_connection(**kwargs):
+    def get_connection(path, log=None, **kwargs):
         """Create a new connection to the database."""
         
-    def init_db(**kwargs):
+    def init_db(path, log=None, **kwargs):
         """Initialize the database."""
 
     def to_sql(table):
         """Return the DDL statements necessary to create the specified table,
         including indices."""
+        
+    def backup(dest):
+        """Backup the database to a location defined by trac.backup_dir"""
 
 
 class DatabaseManager(Component):
@@ -58,10 +67,17 @@ class DatabaseManager(Component):
         [wiki:TracEnvironment#DatabaseConnectionStrings string] for this
         project""")
 
+    backup_dir = Option('trac', 'backup_dir', 'db',
+        """Database backup location""")
+
     timeout = IntOption('trac', 'timeout', '20',
         """Timeout value for database connection, in seconds.
         Use '0' to specify ''no timeout''. ''(Since 0.11)''""")
 
+    debug_sql = BoolOption('trac', 'debug_sql', False,
+        """Show the SQL queries in the Trac log, at DEBUG level.
+        ''(Since 0.11.5)''""")
+
     def __init__(self):
         self._cnx_pool = None
 
@@ -80,21 +96,42 @@ class DatabaseManager(Component):
             self._cnx_pool.shutdown(tid)
             if not tid:
                 self._cnx_pool = None
+                
+    def backup(self, dest=None):
+        """Save a backup of the database.
+
+        @param dest: base filename to write to.
+        Returns the file actually written.
+        """
+        connector, args = self._get_connector()
+        if not dest:
+            backup_dir = self.backup_dir
+            if backup_dir[0] != "/":
+                backup_dir = os.path.join(self.env.path, backup_dir)
+            db_str = self.config.get('trac', 'database')
+            db_name, db_path = db_str.split(":",1)
+            dest_name = '%s.%i.%d.bak' % (db_name, self.env.get_version(),
+                                          int(time.time()))
+            dest = os.path.join(backup_dir, dest_name)
+        else:
+            backup_dir = os.path.dirname(dest)
+        if not os.path.exists(backup_dir):
+            os.makedirs(backup_dir)
+        return connector.backup(dest)
 
     def _get_connector(self): ### FIXME: Make it public?
         scheme, args = _parse_db_str(self.connection_uri)
-        candidates = {}
-        for connector in self.connectors:
-            for scheme_, priority in connector.get_supported_schemes():
-                if scheme_ != scheme:
-                    continue
-                highest = candidates.get(scheme_, (None, 0))[1]
-                if priority > highest:
-                    candidates[scheme] = (connector, priority)
-
-        connector = candidates.get(scheme, [None])[0]
-        if not connector:
+        candidates = [
+            (priority, connector)
+            for connector in self.connectors
+            for scheme_, priority in connector.get_supported_schemes()
+            if scheme_ == scheme
+        ]
+        if not candidates:
             raise TracError('Unsupported database type "%s"' % scheme)
+        priority, connector = max(candidates)
+        if priority < 0:
+            raise TracError(connector.error)
 
         if scheme == 'sqlite':
             # Special case for SQLite to support a path relative to the
@@ -104,6 +141,8 @@ class DatabaseManager(Component):
                 args['path'] = os.path.join(self.env.path,
                                             args['path'].lstrip('/'))
 
+        if self.debug_sql:
+            args['log'] = self.log
         return connector, args
 
 
@@ -116,8 +155,8 @@ def _parse_db_str(db_str):
             host = None
             path = rest
         else:
-            raise TracError('Database connection string must start with '
-                            'scheme:/')
+            raise TracError('Unknown scheme "%s"; database connection string '
+                            'must start with {scheme}:/' % scheme)
     else:
         if not rest.startswith('//'):
             host = None
