@@ -14,16 +14,34 @@
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-import re
+import re, sys, os, time
 
 from trac.core import *
-from trac.db.api import IDatabaseConnector
+from trac.config import Option
+from trac.db.api import IDatabaseConnector, _parse_db_str
 from trac.db.util import ConnectionWrapper
 from trac.util import get_pkginfo
+from trac.util.compat import close_fds
+from trac.util.text import to_unicode
+from trac.util.translation import _
 
-psycopg = None
-PgSQL = None
+has_psycopg = False
+has_pgsql = False
 PGSchemaError = None
+try:
+    import psycopg2 as psycopg
+    import psycopg2.extensions
+    from psycopg2 import ProgrammingError as PGSchemaError
+    psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
+    has_psycopg = True
+except ImportError:
+    try:
+        from pyPgSQL import PgSQL
+        from pyPgSQL.libpq import OperationalError as PGSchemaError
+        has_pgsql = True
+    except ImportError:
+        pass
+
 
 _like_escape_re = re.compile(r'([/_%])')
 
@@ -33,23 +51,28 @@ class PostgreSQLConnector(Component):
 
     implements(IDatabaseConnector)
 
+    pg_dump_path = Option('trac', 'pg_dump_path', 'pg_dump',
+        """Location of pg_dump for Postgres database backups""")
+
     def __init__(self):
         self._version = None
+        self.error = None
 
     def get_supported_schemes(self):
-        return [('postgres', 1)]
+        if not (has_psycopg or has_pgsql):
+            self.error = _("Cannot load Python bindings for PostgreSQL")
+        yield ('postgres', self.error and -1 or 1)
 
-    def get_connection(self, path, user=None, password=None, host=None,
-                       port=None, params={}):
-        global psycopg
-        global PgSQL
-        cnx = PostgreSQLConnection(path, user, password, host, port, params)
+    def get_connection(self, path, log=None, user=None, password=None,
+                       host=None, port=None, params={}):
+        cnx = PostgreSQLConnection(path, log, user, password, host, port,
+                                   params)
         if not self._version:
-            if psycopg:
+            if has_psycopg:
                 self._version = get_pkginfo(psycopg).get('version',
                                                          psycopg.__version__)
                 name = 'psycopg2'
-            elif PgSQL:
+            elif has_pgsql:
                 import pyPgSQL
                 self._version = get_pkginfo(pyPgSQL).get('version',
                                                          pyPgSQL.__version__)
@@ -60,9 +83,10 @@ class PostgreSQLConnector(Component):
             self.env.systeminfo.append((name, self._version))
         return cnx
 
-    def init_db(self, path, user=None, password=None, host=None, port=None,
-                params={}):
-        cnx = self.get_connection(path, user, password, host, port, params)
+    def init_db(self, path, log=None, user=None, password=None, host=None,
+                port=None, params={}):
+        cnx = self.get_connection(path, log, user, password, host, port,
+                                  params)
         cursor = cnx.cursor()
         if cnx.schema:
             cursor.execute('CREATE SCHEMA "%s"' % cnx.schema)
@@ -90,8 +114,49 @@ class PostgreSQLConnector(Component):
         yield '\n'.join(sql)
         for index in table.indices:
             unique = index.unique and 'UNIQUE' or ''
-            yield 'CREATE %s INDEX "%s_%s_idx" ON "%s" ("%s")' % (unique, table.name, 
-                   '_'.join(index.columns), table.name, '","'.join(index.columns))
+            yield 'CREATE %s INDEX "%s_%s_idx" ON "%s" ("%s")' % \
+                    (unique, table.name, 
+                     '_'.join(index.columns), table.name,
+                     '","'.join(index.columns))
+
+    def backup(self, dest_file):
+        try:
+            from subprocess import Popen, PIPE
+        except ImportError:
+            raise TracError('Python >= 2.4 or the subprocess module '
+                            'is required for pre-upgrade backup support')
+        db_url = self.env.config.get('trac', 'database')
+        scheme, db_prop = _parse_db_str(db_url)
+        db_prop.setdefault('params', {})
+        db_name = os.path.basename(db_prop['path'])
+
+        args = [self.pg_dump_path, '-C', '--inserts', '-x', '-Z', '8']
+        if 'user' in db_prop:
+            args.extend(['-U', db_prop['user']])
+        if 'host' in db_prop['params']:
+            host = db_prop['params']['host']
+        else:
+            host = db_prop.get('host', 'localhost')
+        args.extend(['-h', host])
+        if '/' not in host:
+            args.extend(['-p', str(db_prop.get('port', '5432'))])
+
+        if 'schema' in db_prop['params']:
+            args.extend(['-n', '"%s"' % db_prop['params']['schema']])
+
+        dest_file += ".gz"
+        args.extend(['-f', dest_file, db_name])
+
+        environ = os.environ.copy()
+        if 'password' in db_prop:
+            environ['PGPASSWORD'] = str(db_prop['password'])
+        p = Popen(args, env=environ, stderr=PIPE, close_fds=close_fds)
+        errmsg = p.communicate()[1]
+        if p.returncode != 0:
+            raise TracError("Backup attempt failed (%s)" % to_unicode(errmsg))
+        if not os.path.exists(dest_file):
+            raise TracError("Backup attempt failed")
+        return dest_file
 
 
 class PostgreSQLConnection(ConnectionWrapper):
@@ -99,27 +164,13 @@ class PostgreSQLConnection(ConnectionWrapper):
 
     poolable = True
 
-    def __init__(self, path, user=None, password=None, host=None, port=None,
-                 params={}):
+    def __init__(self, path, log=None, user=None, password=None, host=None,
+                 port=None, params={}):
         if path.startswith('/'):
             path = path[1:]
-        # We support both psycopg and PgSQL but prefer psycopg
-        global psycopg
-        global PgSQL
-        global PGSchemaError
-        
-        if not psycopg and not PgSQL:
-            try:
-                import psycopg2 as psycopg
-                import psycopg2.extensions
-                from psycopg2 import ProgrammingError as PGSchemaError
-                psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
-            except ImportError:
-                from pyPgSQL import PgSQL
-                from pyPgSQL.libpq import OperationalError as PGSchemaError
         if 'host' in params:
             host = params['host']
-        if psycopg:
+        if has_psycopg:
             dsn = []
             if path:
                 dsn.append('dbname=' + path)
@@ -133,7 +184,7 @@ class PostgreSQLConnection(ConnectionWrapper):
                 dsn.append('port=' + str(port))
             cnx = psycopg.connect(' '.join(dsn))
             cnx.set_client_encoding('UNICODE')
-        else:
+        elif has_pgsql:
             # Don't use chatty, inefficient server-side cursors.
             # http://pypgsql.sourceforge.net/pypgsql-faq.html#id2787367
             PgSQL.fetchReturnsList = 1
@@ -148,7 +199,7 @@ class PostgreSQLConnection(ConnectionWrapper):
                 cnx.commit()
         except PGSchemaError:
             cnx.rollback()
-        ConnectionWrapper.__init__(self, cnx)
+        ConnectionWrapper.__init__(self, cnx, log)
 
     def cast(self, column, type):
         # Temporary hack needed for the union of selects in the search module
