@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005 Edgewall Software
+# Copyright (C)2005-2009 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -46,5 +46,6 @@ class Column(object):
 class Index(object):
     """Declare an index for a database schema."""
 
-    def __init__(self, columns):
+    def __init__(self, columns, unique=False):
         self.columns = columns
+        self.unique = unique
