@@ -21,7 +21,8 @@ import weakref
 from trac.core import *
 from trac.db.api import IDatabaseConnector
 from trac.db.util import ConnectionWrapper
-from trac.util import get_pkginfo
+from trac.util import get_pkginfo, getuser
+from trac.util.translation import _
 
 _like_escape_re = re.compile(r'([/_%])')
 
@@ -62,6 +63,55 @@ if have_pysqlite == 2:
             return self._rollback_on_error(sqlite.Cursor.executemany, sql,
                                            args or [])
 
+    # EagerCursor taken from the example in pysqlite's repository:
+    #
+    #   http://oss.itsystementwicklung.de/hg/pysqlite/raw-file/0a726720f540/misc/eager.py
+    #
+    # Only change is to subclass it from PyFormatCursor instead of
+    # sqlite.Cursor.
+
+    class EagerCursor(PyFormatCursor):
+        def __init__(self, con):
+            PyFormatCursor.__init__(self, con)
+            self.rows = []
+            self.pos = 0
+
+        def execute(self, *args):
+            PyFormatCursor.execute(self, *args)
+            self.rows = PyFormatCursor.fetchall(self)
+            self.pos = 0
+
+        def fetchone(self):
+            try:
+                row = self.rows[self.pos]
+                self.pos += 1
+                return row
+            except IndexError:
+                return None
+
+        def fetchmany(self, num=None):
+            if num is None:
+                num = self.arraysize
+
+            result = self.rows[self.pos:self.pos+num]
+            self.pos += num
+            return result
+
+        def fetchall(self):
+            result = self.rows[self.pos:]
+            self.pos = len(self.rows)
+            return result
+
+        # needed for the tests (InMemoryDatabase doesn't use IterableCursor)
+
+        def __iter__(self):
+            while True:
+                row = self.fetchone()
+                if not row:
+                    return
+                yield row
+
+
 elif have_pysqlite == 1:
     _ver = sqlite._sqlite.sqlite_version_info()
     sqlite_version = _ver[0] * 10000 + _ver[1] * 100 + _ver[2]
@@ -112,20 +162,33 @@ class SQLiteConnector(Component):
 
     def __init__(self):
         self._version = None
+        self.error = None
 
     def get_supported_schemes(self):
-        return [('sqlite', 1)]
+        if not have_pysqlite:
+            self.error = _("Cannot load Python bindings for SQLite")
+        elif sqlite_version >= 30303:
+            if sqlite.version_info < (1, 0, 7):
+                self.error = _("Need at least PySqlite 1.0.7 or higher")
+            elif sqlite.version_info[0] == 2 and \
+                    sqlite.version_info < (2, 0, 7):
+                self.error = _("Need at least PySqlite 2.0.7 or higher")
+        yield ('sqlite', self.error and -1 or 1)
 
-    def get_connection(self, path, params={}):
+    def get_connection(self, path, log=None, params={}):
         if not self._version:
-            global sqlite_version_string
             self._version = get_pkginfo(sqlite).get(
                 'version', '%d.%d.%s' % sqlite.version_info)
             self.env.systeminfo.extend([('SQLite', sqlite_version_string),
                                         ('pysqlite', self._version)])
-        return SQLiteConnection(path, params)
+            if have_pysqlite == 1:
+                self.log.warning("Support for SQLite v2 and PySqlite 1.0.x "
+                                 "will be dropped in version 0.12, see "
+                                 "http://trac.edgewall.org/wiki/"
+                                 "PySqlite#UpgradingSQLitefrom2.xto3.x")
+        return SQLiteConnection(path, log, params)
 
-    def init_db(cls, path, params={}):
+    def init_db(cls, path, log=None, params={}):
         if path != ':memory:':
             # make the directory to hold the database
             if os.path.exists(path):
@@ -144,14 +207,28 @@ class SQLiteConnector(Component):
     def to_sql(cls, table):
         return _to_sql(table)
 
+    def backup(self, dest_file):
+        """Simple SQLite-specific backup of the database.
+
+        @param dest_file: Destination file basename
+        """
+        import shutil
+        db_str = self.config.get('trac', 'database')
+        db_name = os.path.join(self.env.path, db_str[7:])
+        shutil.copy(db_name, dest_file)
+        if not os.path.exists(dest_file):
+            raise TracError("Backup attempt failed")
+        return dest_file
 
 class SQLiteConnection(ConnectionWrapper):
     """Connection wrapper for SQLite."""
 
-    __slots__ = ['_active_cursors']
-    poolable = have_pysqlite and os.name == 'nt' and sqlite_version >= 30301
+    __slots__ = ['_active_cursors', '_eager']
 
-    def __init__(self, path, params={}):
+    poolable = have_pysqlite and sqlite_version >= 30308 \
+                             and sqlite.version_info >= (2,5,0)
+
+    def __init__(self, path, log=None, params={}):
         assert have_pysqlite > 0
         self.cnx = None
         if path != ':memory:':
@@ -161,15 +238,16 @@ class SQLiteConnection(ConnectionWrapper):
             dbdir = os.path.dirname(path)
             if not os.access(path, os.R_OK + os.W_OK) or \
                    not os.access(dbdir, os.R_OK + os.W_OK):
-                from getpass import getuser
                 raise TracError('The user %s requires read _and_ write ' \
-                                'permission to the database file %s and the ' \
+                                'permissions to the database file %s and the ' \
                                 'directory it is located in.' \
                                 % (getuser(), path))
 
         if have_pysqlite == 2:
             self._active_cursors = weakref.WeakKeyDictionary()
             timeout = int(params.get('timeout', 10.0))
+            self._eager = params.get('cursor', 'eager') == 'eager'
+            # eager is default, can be turned off by specifying ?cursor=
             if isinstance(path, unicode): # needed with 2.4.0
                 path = path.encode('utf-8')
             cnx = sqlite.connect(path, detect_types=sqlite.PARSE_DECLTYPES,
@@ -179,11 +257,11 @@ class SQLiteConnection(ConnectionWrapper):
             timeout = int(params.get('timeout', 10000))
             cnx = sqlite.connect(path, timeout=timeout, encoding='utf-8')
             
-        ConnectionWrapper.__init__(self, cnx)
+        ConnectionWrapper.__init__(self, cnx, log)
 
     if have_pysqlite == 2:
         def cursor(self):
-            cursor = self.cnx.cursor(PyFormatCursor)
+            cursor = self.cnx.cursor((PyFormatCursor, EagerCursor)[self._eager])
             self._active_cursors[cursor] = True
             cursor.cnx = self
             return cursor
