@@ -43,26 +43,47 @@ from trac.web.href import Href
 from trac.wiki.api import WikiSystem
 from trac.wiki.formatter import WikiProcessor, Formatter, extract_link
 
+if has_docutils and StrictVersion(__version__) < StrictVersion('0.6'):
+    # Monkey-patch "raw" role handler in docutils to add a missing check
+    # See docutils bug #2845002 on SourceForge
+    def raw_role(role, rawtext, text, lineno, inliner, options={}, content=[]):
+        if not inliner.document.settings.raw_enabled:
+            msg = inliner.reporter.warning('raw (and derived) roles disabled')
+            prb = inliner.problematic(rawtext, rawtext, msg)
+            return [prb], [msg]
+        return _raw_role(role, rawtext, text, lineno, inliner, options,
+                         content)
+    
+    from docutils.parsers.rst import roles
+    raw_role.options = roles.raw_role.options
+    _raw_role = roles.raw_role
+    roles.raw_role = raw_role
+    roles.register_canonical_role('raw', raw_role)
+
+
 class ReStructuredTextRenderer(Component):
     """
     Renders plain text in reStructuredText format as HTML.
     """
     implements(IHTMLPreviewRenderer)
 
+    can_render = False
+    
+    def __init__(self):
+        if has_docutils:
+            if StrictVersion(__version__) < StrictVersion('0.3.9'):
+                self.log.warning('Docutils version >= %s required, '
+                                 '%s found' % ('0.3.9', __version__))
+            else:
+                self.can_render = True
+                self.env.systeminfo.append(('Docutils', __version__))
+        
     def get_quality_ratio(self, mimetype):
-        if mimetype == 'text/x-rst':
+        if self.can_render and mimetype == 'text/x-rst':
             return 8
         return 0
 
     def render(self, context, mimetype, content, filename=None, rev=None):
-        global has_docutils
-        if not has_docutils:
-            raise TracError(_('Docutils not found'))
-        if StrictVersion(__version__) < StrictVersion('0.3.9'):
-            raise TracError(_('Docutils version >= %(version)s required, '
-                              '%(found)s found', version='0.3.9',
-                              found=__version__))
-
         def trac_get_reference(rawtext, target, text):
             fulltext = text and target+' '+text or target
             link = extract_link(self.env, context, fulltext)
