@@ -40,12 +40,15 @@ class TicketNotificationSystem(Component):
     always_notify_updater = BoolOption('notification', 'always_notify_updater',
                                        'true',
         """Always send notifications to the person who causes the ticket 
-        property change.""")
+        property change and to any previous updater of that ticket.""")
         
     ticket_subject_template = Option('notification', 'ticket_subject_template', 
                                      '$prefix #$ticket.id: $summary',
         """A Genshi text template snippet used to get the notification subject.
-        (since 0.11)""")
+
+        By default, the subject template is `$prefix #$ticket.id: $summary`.
+        `$prefix` being the value of the `smtp_subject_prefix` option.
+        ''(since 0.11)''""")
 
 
 class TicketNotifyEmail(NotifyEmail):
