@@ -34,8 +34,8 @@ from trac.util.presentation import Paginator
 from trac.util.text import to_unicode, unicode_urlencode
 from trac.util.translation import _
 from trac.web.api import IRequestHandler, RequestDone
-from trac.web.chrome import add_ctxtnav, add_link, add_stylesheet, \
-                            INavigationContributor, Chrome
+from trac.web.chrome import add_ctxtnav, add_link, add_notice, \
+                            add_stylesheet, INavigationContributor, Chrome
 from trac.wiki import IWikiSyntaxProvider, WikiParser
 
 
@@ -141,6 +141,7 @@ class ReportModule(Component):
                        "VALUES (%s,%s,%s)", (title, query, description))
         id = db.get_last_id(cursor, 'report')
         db.commit()
+        add_notice(req, _('The report has been created.'))
         req.redirect(req.href.report(id))
 
     def _do_delete(self, req, db, id):
@@ -152,6 +153,7 @@ class ReportModule(Component):
         cursor = db.cursor()
         cursor.execute("DELETE FROM report WHERE id=%s", (id,))
         db.commit()
+        add_notice(req, _('The report {%(id)d} has been deleted.', id=id))
         req.redirect(req.href.report())
 
     def _do_save(self, req, db, id):
@@ -166,6 +168,7 @@ class ReportModule(Component):
             cursor.execute("UPDATE report SET title=%s,query=%s,description=%s "
                            "WHERE id=%s", (title, query, description, id))
             db.commit()
+            add_notice(req, _('Your changes have been saved.'))
         req.redirect(req.href.report(id))
 
     def _render_confirm_delete(self, req, db, id):
@@ -280,6 +283,7 @@ class ReportModule(Component):
             title = '{%i} %s' % (id, title)
 
         report_resource = Resource('report', id)
+        req.perm.require('REPORT_VIEW', report_resource)
         context = Context.from_request(req, report_resource)
         data = {'action': 'view',
                 'report': {'id': id, 'resource': report_resource},
@@ -396,6 +400,7 @@ class ReportModule(Component):
         #  - group rows according to __group__ value, if defined
         #  - group cells the same way headers are grouped
         row_groups = []
+        authorized_results = [] 
         prev_group_value = None
         for row_idx, result in enumerate(results):
             col_idx = 0
@@ -435,6 +440,7 @@ class ReportModule(Component):
             # FIXME: for now, we still need to hardcode the realm in the action
             if resource.realm.upper()+'_VIEW' not in req.perm(resource):
                 continue
+            authorized_results.append(result)
             if email_cells:
                 for cell in email_cells:
                     emails = Chrome(self.env).format_emails(context(resource),
@@ -470,11 +476,11 @@ class ReportModule(Component):
             return 'report.rss', data, 'application/rss+xml'
         elif format == 'csv':
             filename = id and 'report_%s.csv' % id or 'report.csv'
-            self._send_csv(req, cols, results, mimetype='text/csv',
+            self._send_csv(req, cols, authorized_results, mimetype='text/csv',
                            filename=filename)
         elif format == 'tab':
             filename = id and 'report_%s.tsv' % id or 'report.tsv'
-            self._send_csv(req, cols, results, '\t',
+            self._send_csv(req, cols, authorized_results, '\t',
                            mimetype='text/tab-separated-values',
                            filename=filename)
         else:
@@ -638,12 +644,6 @@ class ReportModule(Component):
 
     def _send_csv(self, req, cols, rows, sep=',', mimetype='text/plain',
                   filename=None):
-        req.send_response(200)
-        req.send_header('Content-Type', mimetype + ';charset=utf-8')
-        if filename:
-            req.send_header('Content-Disposition', 'filename=' + filename)
-        req.end_headers()
-
         def iso_time(t):
             return format_time(t, 'iso8601')
 
@@ -664,29 +664,44 @@ class ReportModule(Component):
 
         converters = [col_conversions.get(c.strip('_'), string) for c in cols]
 
-        writer = csv.writer(req, delimiter=sep)
+        out = StringIO()
+        writer = csv.writer(out, delimiter=sep)
         writer.writerow([unicode(c).encode('utf-8') for c in cols])
         for row in rows:
             row = list(row)
             for i in xrange(len(row)):
                 row[i] = converters[i](row[i]).encode('utf-8')
             writer.writerow(row)
+        data = out.getvalue()
 
+        req.send_response(200)
+        req.send_header('Content-Type', mimetype + ';charset=utf-8')
+        req.send_header('Content-Length', len(data))
+        if filename:
+            req.send_header('Content-Disposition', 'filename=' + filename)
+        req.end_headers()
+        req.write(data)
         raise RequestDone
 
     def _send_sql(self, req, id, title, description, sql):
         req.perm.require('REPORT_SQL_VIEW')
+
+        out = StringIO()
+        out.write('-- ## %s: %s ## --\n\n' % (id, title.encode('utf-8')))
+        if description:
+            lines = description.encode('utf-8').splitlines()
+            out.write('-- %s\n\n' % '\n-- '.join(lines))
+        out.write(sql.encode('utf-8'))
+        data = out.getvalue()
+
         req.send_response(200)
         req.send_header('Content-Type', 'text/plain;charset=utf-8')
+        req.send_header('Content-Length', len(data))
         if id:
             req.send_header('Content-Disposition',
                             'filename=report_%s.sql' % id)
         req.end_headers()
-
-        req.write('-- ## %s: %s ## --\n\n' % (id, title))
-        if description:
-            req.write('-- %s\n\n' % '\n-- '.join(description.splitlines()))
-        req.write(sql)
+        req.write(data)
         raise RequestDone
         
     # IWikiSyntaxProvider methods
