@@ -19,7 +19,7 @@
 
 import errno
 import locale
-import os
+import os.path
 import re
 import sys
 import time
@@ -54,6 +54,15 @@ def get_reporter_id(req, arg_name=None):
         return '%s <%s>' % (name, email)
     return name or email or req.authname # == 'anonymous'
 
+if os.name == 'nt':
+    from getpass import getuser
+else:
+    import pwd
+    def getuser():
+        try:
+            return pwd.getpwuid(os.geteuid())[0]
+        except KeyError:
+            return 'unknown'
 
 # -- algorithmic utilities
 
@@ -68,6 +77,144 @@ def embedded_numbers(s):
 
 # -- os utilities
 
+try:
+    WindowsError = WindowsError
+except NameError:
+    class WindowsError(OSError):
+        """Dummy exception replacing WindowsError on non-Windows platforms"""
+
+
+can_rename_open_file = False
+if os.name == 'nt':
+    try:
+        import ctypes
+        MOVEFILE_REPLACE_EXISTING = 0x1
+        MOVEFILE_WRITE_THROUGH = 0x8
+        
+        try:
+            MoveFileTransacted = ctypes.windll.kernel32.MoveFileTransactedA
+            CreateTransaction = ctypes.windll.ktmw32.CreateTransaction
+            CommitTransaction = ctypes.windll.ktmw32.CommitTransaction
+            CloseHandle = ctypes.windll.kernel32.CloseHandle
+            can_rename_open_file = True
+            
+            def rename(src, dst):
+                ta = CreateTransaction(None, 0, 0, 0, 0, 1000, 
+                                       'Trac forced rename')
+                if ta == -1:
+                    raise ctypes.WinError()
+                try:
+                    if not (MoveFileTransacted(src, dst, None, None,
+                                               MOVEFILE_REPLACE_EXISTING
+                                               | MOVEFILE_WRITE_THROUGH, ta) \
+                            and CommitTransaction(ta)):
+                        raise ctypes.WinError()
+                finally:
+                    CloseHandle(ta)
+        except AttributeError:
+            MoveFileEx = ctypes.windll.kernel32.MoveFileExA
+            
+            def rename(src, dst):
+                if not MoveFileEx(src, dst, MOVEFILE_REPLACE_EXISTING
+                                            | MOVEFILE_WRITE_THROUGH):
+                    raise ctypes.WinError()
+    except Exception:
+        import random
+        
+        def rename(src, dst):
+            try:
+                os.rename(src, dst)
+            except OSError, e:
+                if e.errno != errno.EEXIST:
+                    raise
+                old = "%s-%08x" % (dst, random.randint(0, sys.maxint))
+                os.rename(dst, old)
+                os.rename(src, dst)
+                try:
+                    os.unlink(old)
+                except Exception:
+                    pass
+else:
+    rename = os.rename
+    can_rename_open_file = True
+
+
+class AtomicFile(object):
+    """A file that appears atomically with its full content.
+    
+    This file-like object writes to a temporary file in the same directory
+    as the final file. If the file is committed, the temporary file is renamed
+    atomically (on Unix, at least) to its final name. If it is rolled back,
+    the temporary file is removed.
+    """
+    def __init__(self, path, mode='w', bufsize=-1):
+        self._path = path
+        (dir, name) = os.path.split(path)
+        (fd, self._temp) = tempfile.mkstemp(prefix=name + '-', dir=dir)
+        self._file = os.fdopen(fd, mode, bufsize)
+        
+        # Try to preserve permissions and group ownership, but failure
+        # should not be fatal
+        try:
+            st = os.stat(path)
+            if hasattr(os, 'chmod'):
+                os.chmod(self._temp, st.st_mode)
+            if hasattr(os, 'chflags') and hasattr(st, 'st_flags'):
+                os.chflags(self._temp, st.st_flags)
+            if hasattr(os, 'chown'):
+                os.chown(self._temp, -1, st.st_gid)
+        except OSError:
+            pass
+    
+    def __getattr__(self, name):
+        return getattr(self._file, name)
+    
+    def commit(self):
+        if self._file is None:
+            return
+        try:
+            f, self._file = self._file, None
+            f.close()
+            rename(self._temp, self._path)
+        except Exception:
+            os.unlink(self._temp)
+            raise
+    
+    def rollback(self):
+        if self._file is None:
+            return
+        try:
+            f, self._file = self._file, None
+            f.close()
+        finally:
+            try:
+                os.unlink(self._temp)
+            except:
+                pass
+    
+    close = commit
+    __del__ = rollback
+
+
+def read_file(path, mode='r'):
+    """Read a file and return its content."""
+    f = open(path, mode)
+    try:
+        return f.read()
+    finally:
+        f.close()
+
+
+def create_file(path, data='', mode='w'):
+    """Create a new file with the given data."""
+    f = open(path, mode)
+    try:
+        if data:
+            f.write(data)
+    finally:
+        f.close()
+
+
 def create_unique_file(path):
     """Create a new file. An index is added if the path exists"""
     parts = os.path.splitext(path)
@@ -102,7 +249,7 @@ class NaivePopen:
     def __init__(self, command, input=None, capturestderr=None):
         outfile = tempfile.mktemp()
         command = '( %s ) > %s' % (command, outfile)
-        if input:
+        if input is not None:
             infile = tempfile.mktemp()
             tmp = open(infile, 'w')
             tmp.write(input)
@@ -387,13 +534,23 @@ class Ranges(object):
     [1, 2, 3, 5, 6, 7, 8, 9]
 
     ''Code contributed by Tim Hatch''
+
+    Reversed ranges are ignored, unless the Ranges has the `reorder` property 
+    set.
+
+    >>> str(Ranges("20-10"))
+    ''
+    >>> str(Ranges("20-10", reorder=True))
+    '10-20'
+
     """
 
     RE_STR = r"""\d+(?:[-:]\d+)?(?:,\d+(?:[-:]\d+)?)*"""
     
-    def __init__(self, r=None):
+    def __init__(self, r=None, reorder=False):
         self.pairs = []
         self.a = self.b = None
+        self.reorder = reorder
         self.appendrange(r)
 
     def appendrange(self, r):
@@ -408,6 +565,8 @@ class Ranges(object):
                 a, b = int(x), int(x)
             if b >= a:
                 p.append((a, b))
+            elif self.reorder:
+                p.append((b, a))
         self._reduce()
 
     def _reduce(self):
@@ -480,11 +639,71 @@ class Ranges(object):
         else:
             return 0
 
+    def truncate(self, max):
+        """Truncate the Ranges by setting a maximal allowed value.
+
+        Note that this `max` can be a value in a gap, so the only guarantee 
+        is that `self.b` will be lesser than or equal to `max`.
+
+        >>> r = Ranges("10-20,25-45")
+        >>> str(r.truncate(30))
+        '10-20,25-30'
+
+        >>> str(r.truncate(22))
+        '10-20'
+
+        >>> str(r.truncate(10))
+        '10'
+        """
+        r = Ranges()
+        r.a, r.b, r.reorder = self.a, self.b, self.reorder
+        r.pairs = []
+        for a, b in self.pairs:
+            if a <= max:
+                if b > max:
+                    r.pairs.append((a, max))
+                    r.b = max
+                    break
+                r.pairs.append((a, b))
+            else:
+                break
+        return r
+
+
+def to_ranges(revs):
+    """Converts a list of revisions to a minimal set of ranges.
+    
+    >>> to_ranges([2, 12, 3, 6, 9, 1, 5, 11])
+    '1-3,5-6,9,11-12'
+    >>> to_ranges([])
+    ''
+    """
+    ranges = []
+    begin = end = None
+    def store():
+        if end == begin:
+            ranges.append(str(begin))
+        else:
+            ranges.append('%d-%d' % (begin, end))
+    for rev in sorted(revs):
+        if begin is None:
+            begin = end = rev
+        elif rev == end + 1:
+            end = rev
+        else:
+            store()
+            begin = end = rev
+    if begin is not None:
+        store()
+    return ','.join(ranges)
+
 def content_disposition(type, filename=None):
     """Generate a properly escaped Content-Disposition header"""
-    if isinstance(filename, unicode):
-        filename = filename.encode('utf-8')
-    return type + '; filename=' + quote(filename, safe='')
+    if filename is not None:
+        if isinstance(filename, unicode):
+            filename = filename.encode('utf-8')
+        type += '; filename=' + quote(filename, safe='')
+    return type
 
 def pairwise(iterable):
     """s -> (s0,s1), (s1,s2), (s2, s3), ...
