@@ -29,7 +29,7 @@ from genshi.builder import tag
 
 from trac.config import Option, BoolOption, IntOption
 from trac.core import *
-from trac.mimeview import Mimeview, is_binary, Context
+from trac.mimeview import Context, Mimeview
 from trac.perm import IPermissionRequestor
 from trac.resource import Resource, ResourceNotFound
 from trac.search import ISearchSource, search_to_sql, shorten_result
@@ -37,12 +37,13 @@ from trac.timeline.api import ITimelineEventProvider
 from trac.util import embedded_numbers, content_disposition
 from trac.util.compat import any, sorted, groupby
 from trac.util.datefmt import pretty_timedelta, utc
-from trac.util.text import unicode_urlencode, shorten_line, CRLF
+from trac.util.text import exception_to_unicode, unicode_urlencode, \
+                           shorten_line, CRLF
 from trac.util.translation import _
 from trac.versioncontrol import Changeset, Node, NoSuchChangeset
-from trac.versioncontrol.diff import get_diff_options, diff_blocks, unified_diff
-from trac.versioncontrol.web_ui.browser import BrowserModule, \
-                                               DefaultPropertyRenderer
+from trac.versioncontrol.diff import get_diff_options, diff_blocks, \
+                                     unified_diff
+from trac.versioncontrol.web_ui.browser import BrowserModule
 from trac.web import IRequestHandler, RequestDone
 from trac.web.chrome import add_ctxtnav, add_link, add_script, add_stylesheet, \
                             prevnext_nav, INavigationContributor, Chrome
@@ -86,10 +87,7 @@ class DefaultPropertyDiffRenderer(Component):
     implements(IPropertyDiffRenderer)
 
     def match_property_diff(self, name):
-        # Support everything but hidden properties.
-        hidden_properties = DefaultPropertyRenderer(self.env).hidden_properties
-        return name not in hidden_properties and 1 or 0
-
+        return 1
 
     def render_property_diff(self, name, old_context, old_props,
                              new_context, new_props, options):
@@ -472,7 +470,7 @@ class ChangesetModule(Component):
             new_ctx = Context.from_request(req, new_source)
             changed_properties = []
             if old_props != new_props:
-                for k,v in old_props.items():
+                for k, v in sorted(old_props.items()):
                     new = old = diff = None
                     if not k in new_props:
                         old = v # won't be displayed, no need to render it
@@ -487,12 +485,13 @@ class ChangesetModule(Component):
                     if new or old or diff:
                         changed_properties.append({'name': k, 'old': old,
                                                    'new': new, 'diff': diff})
-                for k,v in new_props.items():
+                for k, v in sorted(new_props.items()):
                     if not k in old_props:
                         new = browser.render_property(k, 'changeset',
                                                       new_ctx, new_props)
-                        changed_properties.append({'name': k, 'new': new,
-                                                   'old': None})
+                        if new is not None:
+                            changed_properties.append({'name': k, 'new': new,
+                                                       'old': None})
             return changed_properties
 
         def _estimate_changes(old_node, new_node):
@@ -506,15 +505,18 @@ class ChangesetModule(Component):
             The list is empty when no differences between comparable files
             are detected, but the return value is None for non-comparable files.
             """
-            old_content = old_node.get_content().read()
-            if is_binary(old_content):
-                return None
-
-            new_content = new_node.get_content().read()
-            if is_binary(new_content):
-                return None
-
             mview = Mimeview(self.env)
+            if mview.is_binary(old_node.content_type, old_node.path):
+                return None
+            if mview.is_binary(new_node.content_type, new_node.path):
+                return None
+            old_content = old_node.get_content().read()
+            if mview.is_binary(content=old_content):
+                return None
+            new_content = new_node.get_content().read()
+            if mview.is_binary(content=new_content):
+                return None
+
             old_content = mview.to_unicode(old_content, old_node.content_type)
             new_content = mview.to_unicode(new_content, new_node.content_type)
 
@@ -622,7 +624,10 @@ class ChangesetModule(Component):
             stream = Chrome(self.env).render_template(req, 'changeset.html',
                                                       data, fragment=True)
             content = stream.select('//div[@id="content"]')
-            req.write(content.render('xhtml'))
+            str_content = content.render('xhtml')
+            req.send_header('Content-Length', len(str_content))
+            req.end_headers()
+            req.write(str_content)
             raise RequestDone
 
         return data
@@ -632,10 +637,10 @@ class ChangesetModule(Component):
         req.send_response(200)
         req.send_header('Content-Type', 'text/x-patch;charset=utf-8')
         req.send_header('Content-Disposition',
-                        content_disposition('inline;', filename + '.diff'))
-        req.end_headers()
-
+                        content_disposition('inline', filename + '.diff'))
+        buf = StringIO()
         mimeview = Mimeview(self.env)
+
         for old_node, new_node, kind, change in repos.get_changes(
             new_path=data['new_path'], new_rev=data['new_rev'],
             old_path=data['old_path'], old_rev=data['old_rev']):
@@ -650,15 +655,19 @@ class ChangesetModule(Component):
             mimeview = Mimeview(self.env)
 
             if old_node:
+                if mimeview.is_binary(old_node.content_type, old_node.path):
+                    continue
                 old_content = old_node.get_content().read()
-                if is_binary(old_content):
+                if mimeview.is_binary(content=old_content):
                     continue
                 old_node_info = (old_node.path, old_node.rev)
                 old_content = mimeview.to_unicode(old_content,
                                                   old_node.content_type)
             if new_node:
+                if mimeview.is_binary(new_node.content_type, new_node.path):
+                    continue
                 new_content = new_node.get_content().read()
-                if is_binary(new_content):
+                if mimeview.is_binary(content=new_content):
                     continue
                 new_node_info = (new_node.path, new_node.rev)
                 new_path = new_node.path
@@ -680,16 +689,21 @@ class ChangesetModule(Component):
                 ignore_space = options.get('ignorewhitespace')
                 if not old_node_info[0]:
                     old_node_info = new_node_info # support for 'A'dd changes
-                req.write('Index: ' + new_path + CRLF)
-                req.write('=' * 67 + CRLF)
-                req.write('--- %s (revision %s)' % old_node_info + CRLF)
-                req.write('+++ %s (revision %s)' % new_node_info + CRLF)
+                buf.write('Index: ' + new_path + CRLF)
+                buf.write('=' * 67 + CRLF)
+                buf.write('--- %s\t(revision %s)' % old_node_info + CRLF)
+                buf.write('+++ %s\t(revision %s)' % new_node_info + CRLF)
                 for line in unified_diff(old_content.splitlines(),
                                          new_content.splitlines(), context,
                                          ignore_blank_lines=ignore_blank_lines,
                                          ignore_case=ignore_case,
                                          ignore_space_changes=ignore_space):
-                    req.write(line + CRLF)
+                    buf.write(line + CRLF)
+                    
+        diff_str = buf.getvalue().encode('utf-8')
+        req.send_header('Content-Length', len(diff_str))
+        req.end_headers()
+        req.write(diff_str)
         raise RequestDone
 
     def _render_zip(self, req, filename, repos, data):
@@ -698,7 +712,7 @@ class ChangesetModule(Component):
         req.send_response(200)
         req.send_header('Content-Type', 'application/zip')
         req.send_header('Content-Disposition',
-                        content_disposition('inline;', filename + '.zip'))
+                        content_disposition('inline', filename + '.zip'))
 
         from zipfile import ZipFile, ZipInfo, ZIP_DEFLATED
 
@@ -713,18 +727,17 @@ class ChangesetModule(Component):
                 zipinfo.filename = new_node.path.strip('/').encode('utf-8')
                 # Note: unicode filenames are not supported by zipfile.
                 # UTF-8 is not supported by all Zip tools either,
-                # but as some does, I think UTF-8 is the best option here.
+                # but as some do, I think UTF-8 is the best option here.
                 zipinfo.date_time = new_node.last_modified.utctimetuple()[:6]
                 zipinfo.external_attr = 0644 << 16L # needed since Python 2.5
                 zipinfo.compress_type = ZIP_DEFLATED
                 zipfile.writestr(zipinfo, new_node.get_content().read())
         zipfile.close()
 
-        buf.seek(0, 2) # be sure to be at the end
-        req.send_header("Content-Length", buf.tell())
+        zip_str = buf.getvalue()
+        req.send_header("Content-Length", len(zip_str))
         req.end_headers()
-
-        req.write(buf.getvalue())
+        req.write(zip_str)
         raise RequestDone
 
     def title_for_diff(self, data):
@@ -740,17 +753,25 @@ class ChangesetModule(Component):
     def render_property_diff(self, name, old_node, old_props,
                              new_node, new_props, options):
         """Renders diffs of a node property to HTML."""
+        if name in BrowserModule(self.env).hidden_properties:
+            return
         candidates = []
         for renderer in self.property_diff_renderers:
             quality = renderer.match_property_diff(name)
             if quality > 0:
                 candidates.append((quality, renderer))
-        if candidates:
-            renderer = sorted(candidates, reverse=True)[0][1]
-            return renderer.render_property_diff(name, old_node, old_props,
-                                                 new_node, new_props, options)
-        else:
-            return None
+        candidates.sort()
+        candidates.reverse()
+        for (quality, renderer) in candidates:
+            try:
+                return renderer.render_property_diff(name, old_node, old_props,
+                                                     new_node, new_props,
+                                                     options)
+            except Exception, e:
+                self.log.warning('Diff rendering failed for property %s with '
+                                 'renderer %s: %s', name,
+                                 renderer.__class__.__name__,
+                                 exception_to_unicode(e, traceback=True))
 
     def _get_location(self, files):
         """Return the deepest common path for the given files.
@@ -998,14 +1019,16 @@ class AnyDiffModule(Component):
                     return embedded_numbers(entry.name)
                 return entry.isfile, name_order(entry)
 
-            html = tag.ul(
+            elem = tag.ul(
                 [tag.li(is_dir and tag.b(path) or path)
                  for e in sorted(node.get_entries(), key=kind_order)
                  for is_dir, path in [(e.isdir, '/' + e.path.lstrip('/'))]
                  if e.name.lower().startswith(prefix)]
             )
 
-            req.write(html.generate().render('xhtml'))
+            xhtml = elem.generate().render('xhtml')
+            req.send_header('Content-Length', len(xhtml))
+            req.write(xhtml)
             return
 
         # -- retrieve arguments
