@@ -579,8 +579,12 @@ class Request(object):
         if __debug__: _debug(1, 'protocolStatus = %d, appStatus = %d' %
                              (protocolStatus, appStatus))
 
-        self._flush()
-        self._end(appStatus, protocolStatus)
+        try: 
+            self._flush() 
+            self._end(appStatus, protocolStatus) 
+        except socket.error, e: 
+            if e[0] != errno.EPIPE: 
+                raise 
 
     def _end(self, appStatus=0L, protocolStatus=FCGI_REQUEST_COMPLETE):
         self._conn.end_request(self, appStatus, protocolStatus)
@@ -805,7 +809,7 @@ class Connection(object):
         outrec = Record(FCGI_UNKNOWN_TYPE)
         outrec.contentData = struct.pack(FCGI_UnknownTypeBody, inrec.type)
         outrec.contentLength = FCGI_UnknownTypeBody_LEN
-        self.writeRecord(rec)
+        self.writeRecord(outrec)
         
 class MultiplexedConnection(Connection):
     """
@@ -1137,7 +1141,7 @@ class WSGIServer(Server):
     FastCGI server that supports the Web Server Gateway Interface. See
     <http://www.python.org/peps/pep-0333.html>.
     """
-    def __init__(self, application, environ=None, umask=None,
+    def __init__(self, application, environ=None,
                  multithreaded=True, **kw):
         """
         environ, if present, must be a dictionary-like object. Its
