@@ -180,6 +180,7 @@ class Request(object):
             'incookie': Request._parse_cookies,
             '_inheaders': Request._parse_headers
         }
+        self.redirect_listeners = []
 
         self.base_url = self.environ.get('trac.base_url')
         if not self.base_url:
@@ -223,6 +224,13 @@ class Request(object):
     server_port = property(fget=lambda self: int(self.environ['SERVER_PORT']),
                            doc='Port number the server is bound to')
 
+    def add_redirect_listener(self, listener):
+        """Add a callable to be called prior to executing a redirect.
+        
+        The callable is passed the arguments to the `redirect()` call.
+        """
+        self.redirect_listeners.append(listener)
+
     def get_header(self, name):
         """Return the value of the specified HTTP header, or `None` if there's
         no such header in the request.
@@ -282,6 +290,7 @@ class Request(object):
             self.send_header('ETag', etag)
         else:
             self.send_response(304)
+            self.send_header('Content-Length', 0)
             self.end_headers()
             raise RequestDone
 
@@ -290,8 +299,10 @@ class Request(object):
         `url` may be relative or absolute, relative URLs will be translated
         appropriately.
         """
-        if self.session:
-            self.session.save() # has to be done before the redirect is sent
+        for listener in self.redirect_listeners:
+            listener(self, url, permanent)
+        
+        self.session.save() # has to be done before the redirect is sent
 
         if permanent:
             status = 301 # 'Moved Permanently'
@@ -308,13 +319,12 @@ class Request(object):
 
         self.send_header('Location', url)
         self.send_header('Content-Type', 'text/plain')
+        self.send_header('Content-Length', 0)
         self.send_header('Pragma', 'no-cache')
         self.send_header('Cache-control', 'no-cache')
         self.send_header('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT')
         self.end_headers()
 
-        if self.method != 'HEAD':
-            self.write('Redirecting...')
         raise RequestDone
 
     def display(self, template, content_type='text/html', status=200):
@@ -406,6 +416,7 @@ class Request(object):
         last_modified = http_date(mtime)
         if last_modified == self.get_header('If-Modified-Since'):
             self.send_response(304)
+            self.send_header('Content-Length', 0)
             self.end_headers()
             raise RequestDone
 
@@ -471,7 +482,7 @@ class Request(object):
         # FieldStorage where QUERY_STRING is no longer ignored for POST
         # requests. We'll keep the pre 2.6 behaviour for now...
         if self.method == 'POST':
-            qs_on_post = self.environ.pop('QUERY_STRING')
+            qs_on_post = self.environ.pop('QUERY_STRING', '')
         fs = cgi.FieldStorage(fp, environ=self.environ, keep_blank_values=True)
         if self.method == 'POST':
             self.environ['QUERY_STRING'] = qs_on_post
