@@ -295,8 +295,8 @@ class BasicAuthentication(PasswordFileAuthentication):
                     return user
 
         start_response('401 Unauthorized',
-                       [('WWW-Authenticate', 'Basic realm="%s"'
-                         % self.realm)])('')
+                       [('WWW-Authenticate', 'Basic realm="%s"' % self.realm),
+                        ('Content-Length', '0')])('')
 
 
 class DigestAuthentication(PasswordFileAuthentication):
@@ -351,7 +351,8 @@ class DigestAuthentication(PasswordFileAuthentication):
         start_response('401 Unauthorized',
                        [('WWW-Authenticate',
                         'Digest realm="%s", nonce="%s", qop="auth", stale="%s"'
-                        % (self.realm, nonce, stale))])('')
+                        % (self.realm, nonce, stale)),
+                        ('Content-Length', '0')])('')
 
     def do_auth(self, environ, start_response):
         header = environ.get('HTTP_AUTHORIZATION')
