@@ -15,7 +15,8 @@
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
 import datetime
-import os
+import itertools
+import os.path
 import pkg_resources
 import pprint
 import re
@@ -40,7 +41,7 @@ from trac.resource import *
 from trac.util import compat, get_reporter_id, presentation, get_pkginfo, \
                       get_module_path, translation
 from trac.util.compat import partial, set
-from trac.util.html import plaintext
+from trac.util.html import escape, plaintext
 from trac.util.text import pretty_size, obfuscate_email_address, \
                            shorten_line, unicode_quote_plus, to_unicode, \
                            javascript_quote
@@ -185,6 +186,14 @@ def prevnext_nav(req, label, uplabel=None):
                      Markup(' &rarr;'), class_='missing'))
 
 
+def _save_messages(req, url, permanent):
+    """Save warnings and notices in case of redirect, so that they can
+    be displayed after the redirect."""
+    for type_ in ['warnings', 'notices']:
+        for (i, message) in enumerate(req.chrome[type_]):
+            req.session['chrome.%s.%d' % (type_, i)] = escape(message)
+
+
 class INavigationContributor(Interface):
     """Extension point interface for components that contribute items to the
     navigation.
@@ -252,7 +261,7 @@ class Chrome(Component):
         
         (''since 0.11'')""")
 
-    auto_reload = Option('trac', 'auto_reload', False,
+    auto_reload = BoolOption('trac', 'auto_reload', False,
         """Automatically reload template files after modification.""")
 
     htdocs_location = Option('trac', 'htdocs_location', '',
@@ -289,6 +298,12 @@ class Chrome(Component):
         """Show email addresses instead of usernames. If false, we obfuscate
         email addresses (''since 0.11'').""")
 
+    never_obfuscate_mailto = BoolOption('trac', 'never_obfuscate_mailto', 
+        'false',
+        """Never obfuscate `mailto:` links explicitly written in the wiki, 
+        even if `show_email_addresses` is false or the user has not the 
+        EMAIL_VIEW permission (''since 0.11.6'').""")
+
     show_ip_addresses = BoolOption('trac', 'show_ip_addresses', 'false',
         """Show IP addresses for resource edits (e.g. wiki).
         (''since 0.11.3'').""")
@@ -345,14 +360,17 @@ class Chrome(Component):
             if not os.path.exists(templates_dir):
                 os.mkdir(templates_dir)
 
-            fileobj = open(os.path.join(templates_dir, 'site.html'), 'w')
-            try:
-                fileobj.write("""<html xmlns="http://www.w3.org/1999/xhtml"
+            if not self.shared_templates_dir or not os.path.exists(
+                        os.path.join(self.shared_templates_dir, "site.html")):
+                fileobj = open(os.path.join(templates_dir, 'site.html'), 'w')
+                try:
+                    fileobj.write("""\
+<html xmlns="http://www.w3.org/1999/xhtml"
       xmlns:py="http://genshi.edgewall.org/" py:strip="">
   <!--! Custom match templates go here -->
 </html>""")
-            finally:
-                fileobj.close()
+                finally:
+                    fileobj.close()
 
     def environment_needs_upgrade(self, db):
         return False
@@ -484,7 +502,7 @@ class Chrome(Component):
                     href = category_section.get(name + '.href')
                     if href:
                         if href.startswith('/'):
-                            href = req.href() + href
+                            href = req.href + href
                         if label:
                             item = tag.a(label) # create new label
                         elif not item:
@@ -524,6 +542,9 @@ class Chrome(Component):
         # Default theme file
         chrome['theme'] = 'theme.html'
 
+        # Avoid recursion by registering as late as possible (#8583)
+        req.add_redirect_listener(_save_messages)
+
         return chrome
 
     def get_icon_data(self, req):
@@ -704,6 +725,16 @@ class Chrome(Component):
         method = {'text/html': 'xhtml',
                   'text/plain': 'text'}.get(content_type, 'xml')
 
+        if method == "xhtml":
+            # Retrieve post-redirect messages saved in session
+            for type_ in ['warnings', 'notices']:
+                try:
+                    for i in itertools.count():
+                        message = req.session.pop('chrome.%s.%d' % (type_, i))
+                        req.chrome[type_].append(Markup(message))
+                except KeyError:
+                    pass
+
         template = self.load_template(filename, method=method)
         data = self.populate_data(req, data)
 
@@ -725,7 +756,7 @@ class Chrome(Component):
         if doctype:
             if req.form_token:
                 stream |= self._add_form_token(req.form_token)
-            if not req.session or not int(req.session.get('accesskeys', 0)):
+            if not int(req.session.get('accesskeys', 0)):
                 stream |= self._strip_accesskeys
 
         links = req.chrome.get('links')
