@@ -121,7 +121,8 @@ class RequestDispatcher(Component):
         """Name of the component that handles requests to the base URL.
         
         Options include `TimelineModule`, `RoadmapModule`, `BrowserModule`,
-        `QueryModule`, `ReportModule` and `TicketModule` (''since 0.9'').""")
+        `QueryModule`, `ReportModule`, `TicketModule` and `WikiModule`. The
+        default is `WikiModule`. (''since 0.9'')""")
 
     default_timezone = Option('trac', 'default_timezone', '',
         """The default timezone to use""")
@@ -181,7 +182,7 @@ class RequestDispatcher(Component):
                         target = req.path_info.rstrip('/').encode('utf-8')
                         if req.query_string:
                             target += '?' + req.query_string
-                        req.redirect(req.href() + target, permanent=True)
+                        req.redirect(req.href + target, permanent=True)
                     raise HTTPNotFound('No handler matched request to %s',
                                        req.path_info)
 
@@ -209,8 +210,7 @@ class RequestDispatcher(Component):
                         template, content_type = \
                                   self._post_process_request(req, *resp)
                         # Give the session a chance to persist changes
-                        if req.session:
-                            req.session.save()
+                        req.session.save()
                         req.display(template, content_type or 'text/html')
                     else: # Genshi
                         template, data, content_type = \
@@ -226,9 +226,7 @@ class RequestDispatcher(Component):
                             output = chrome.render_template(req, template,
                                                             data, content_type)
                             # Give the session a chance to persist changes
-                            if req.session:
-                                req.session.save()
-
+                            req.session.save()
                             req.send(output, content_type or 'text/html')
                 else:
                     self._post_process_request(req)
@@ -377,8 +375,11 @@ def dispatch_request(environ, start_response):
                 env_path = get_environments(environ).get(env_name)
 
             if not env_path or not os.path.isdir(env_path):
-                start_response('404 Not Found', [])
-                return ['Environment not found']
+                errmsg = 'Environment not found'
+                start_response('404 Not Found', 
+                               [('Content-Type', 'text/plain'),
+                                ('Content-Length', str(len(errmsg)))])
+                return [errmsg]
 
     if not env_path:
         raise EnvironmentError('The environment options "TRAC_ENV" or '
@@ -393,7 +394,20 @@ def dispatch_request(environ, start_response):
         env = open_environment(env_path, use_cache=not run_once)
         if env.base_url_for_redirect:
             environ['trac.base_url'] = env.base_url
+
+        # Web front-end type and version information
         if not hasattr(env, 'webfrontend'):
+            mod_wsgi_version = environ.get('mod_wsgi.version')
+            if mod_wsgi_version:
+                mod_wsgi_version = (
+                        "%s (WSGIProcessGroup %s WSGIApplicationGroup %s)" % 
+                        ('.'.join([str(x) for x in mod_wsgi_version]),
+                         environ.get('mod_wsgi.process_group'),
+                         environ.get('mod_wsgi.application_group') or 
+                         '%{GLOBAL}'))
+                environ.update({
+                    'trac.web.frontend': 'mod_wsgi',
+                    'trac.web.version': mod_wsgi_version})
             env.webfrontend = environ.get('trac.web.frontend')
             if env.webfrontend:
                 env.systeminfo.append((env.webfrontend, 
@@ -412,9 +426,10 @@ def dispatch_request(environ, start_response):
             # Note: enable the '##' lines as soon as there's a suspicion
             #       of memory leak due to uncollectable objects (typically
             #       objects with a __del__ method caught in a cycle)
+            #
             ##gc.set_debug(gc.DEBUG_UNCOLLECTABLE)
             unreachable = gc.collect()
-            env.log.debug("%d unreachable objects found.", unreachable)
+            ##env.log.debug("%d unreachable objects found.", unreachable)
             ##uncollectable = len(gc.garbage)
             ##if uncollectable:
             ##    del gc.garbage[:]
@@ -548,6 +563,7 @@ def send_project_index(environ, start_response, parent_dir=None,
                 env = open_environment(env_path,
                                        use_cache=not environ['wsgi.run_once'])
                 proj = {
+                    'env': env,
                     'name': env.project_name,
                     'description': env.project_description,
                     'href': href(env_name)
