@@ -102,9 +102,11 @@ class TracEnvironMiddleware(object):
 
 class TracHTTPServer(ThreadingMixIn, WSGIServer):
 
-    def __init__(self, server_address, application, env_parent_dir, env_paths):
+    def __init__(self, server_address, application, env_parent_dir, env_paths,
+                 use_http_11=False):
+        request_handlers = (TracHTTPRequestHandler, TracHTTP11RequestHandler)
         WSGIServer.__init__(self, server_address, application,
-                            request_handler=TracHTTPRequestHandler)
+                            request_handler=request_handlers[bool(use_http_11)])
 
 
 class TracHTTPRequestHandler(WSGIRequestHandler):
@@ -115,6 +117,9 @@ class TracHTTPRequestHandler(WSGIRequestHandler):
         # Disable reverse name lookups
         return self.client_address[:2][0]
 
+class TracHTTP11RequestHandler(TracHTTPRequestHandler):
+    protocol_version = 'HTTP/1.1'
+
 
 def main():
     from optparse import OptionParser, OptionValueError
@@ -141,6 +146,12 @@ def main():
                                    % (opt_str, '|'.join(valid_values), value))
         setattr(parser.values, option.dest, value)
 
+    def _octal(option, opt_str, value, parser):
+        try:
+            setattr(parser.values, option.dest, int(value, 8))
+        except ValueError:
+            raise OptionValueError('Invalid octal umask value: %r' % value)
+    
     parser.add_option('-a', '--auth', action='callback', type='string',
                       metavar='DIGESTAUTH', callback=_auth_callback,
                       callback_args=(DigestAuthentication,),
@@ -160,7 +171,11 @@ def main():
                       help='http|scgi|ajp')
     parser.add_option('-q', '--unquote', action='store_true',
                       dest='unquote',
-                      help='unquote PATH_INFO (may be needed when using ajp')
+                      help='unquote PATH_INFO (may be needed when using ajp)')
+    parser.add_option('--http10', action='store_false', dest='http11',
+                      help='use HTTP/1.0 protocol version (default)')
+    parser.add_option('--http11', action='store_true', dest='http11',
+                      help='use HTTP/1.1 protocol version instead of HTTP/1.0')
     parser.add_option('-e', '--env-parent-dir', action='store',
                       dest='env_parent_dir', metavar='PARENTDIR',
                       help='parent directory of the project environments')
@@ -183,10 +198,10 @@ def main():
         parser.add_option('--pidfile', action='store',
                           dest='pidfile',
                           help='When daemonizing, file to which to write pid')
-        parser.add_option('--umask', action='store', type='int', dest='umask',
-                          metavar='MASK',
+        parser.add_option('--umask', action='callback', type='string',
+                          dest='umask', metavar='MASK', callback=_octal,
                           help='When daemonizing, file mode creation mask '
-                          'to use (default 022)')
+                          'to use, in octal notation (default 022)')
 
     parser.set_defaults(port=None, hostname='', base_path='', daemonize=False,
                         protocol='http', umask=022)
@@ -237,7 +252,9 @@ def main():
     if options.protocol == 'http':
         def serve():
             httpd = TracHTTPServer(server_address, wsgi_app,
-                                   options.env_parent_dir, args)
+                                   options.env_parent_dir, args,
+                                   use_http_11=options.http11)
+
             print 'Server starting in PID %i.' % os.getpid()
             addr, port = server_address
             if not addr or addr == '0.0.0.0':
@@ -245,6 +262,8 @@ def main():
                        % (port, port, base_path)
             else:
                 print 'Serving on http://%s:%s/%s' % (addr, port, base_path)
+            if options.http11:
+                print 'Using HTTP/1.1 protocol version'
             httpd.serve_forever()
     elif options.protocol in ('scgi', 'ajp'):
         def serve():
